.g-item-view-large-image
  margin-top 20px

.li-metadata-tabs
  margin-top 10px
  overflow-x hidden
  .yaml
    display block
    unicode-bidi embed
    font-family monospace
    white-space pre-wrap
  td
    max-width 50vw
    td
      max-width 33vw

a.g-widget-auximage
  display inline-block
  padding 5px
  .g-widget-auximage-title
    color black
    font-weight bold
    font-size 1.2em
